"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.MarkdownEditorView = exports.cnEditorComponent = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const react_error_boundary_1 = require("react-error-boundary");
const react_use_1 = require("react-use");
const classname_1 = require("../classname");
const bundle_1 = require("../i18n/bundle");
const logger_1 = require("../logger");
const hooks_1 = require("../react-utils/hooks");
const toaster_1 = require("../react-utils/toaster");
const useSticky_1 = require("../react-utils/useSticky");
const platform_1 = require("../utils/platform");
const HorizontalDrag_1 = require("./HorizontalDrag");
const MarkupEditorView_1 = require("./MarkupEditorView");
const SplitModeView_1 = require("./SplitModeView");
const WysiwygEditorView_1 = require("./WysiwygEditorView");
const markup_1 = require("./config/markup");
const wysiwyg_1 = require("./config/wysiwyg");
const context_1 = require("./context");
const settings_1 = require("./settings");
const sticky_1 = require("./sticky");
// eslint-disable-line import/order
exports.cnEditorComponent = (0, classname_1.cn)('editor-component');
const b = exports.cnEditorComponent;
exports.MarkdownEditorView = react_1.default.forwardRef((props, ref) => {
    var _a, _b;
    const divRef = (0, react_use_1.useEnsuredForwardedRef)(ref);
    const [isMounted, setIsMounted] = (0, react_1.useState)(false);
    (0, react_1.useEffect)(() => {
        setIsMounted(true);
    }, []);
    const [showPreview, , unsetShowPreview, toggleShowPreview] = (0, hooks_1.useBooleanState)(false);
    const context = (0, context_1.useMarkdownEditorContext)();
    const editor = ((_a = props.editor) !== null && _a !== void 0 ? _a : context);
    if (!editor)
        throw new Error('[MarkdownEditorView]: an instance of the editor must be passed through the props or context');
    const { autofocus, className, settingsVisible = true, markupToolbarConfig = markup_1.mToolbarConfigByPreset[editor.preset], wysiwygToolbarConfig = wysiwyg_1.wToolbarConfigByPreset[editor.preset], markupHiddenActionsConfig = markup_1.mHiddenDataByPreset[editor.preset], wysiwygHiddenActionsConfig = wysiwyg_1.wHiddenDataByPreset[editor.preset], toaster, stickyToolbar, } = props;
    const rerender = (0, react_use_1.useUpdate)();
    react_1.default.useLayoutEffect(() => {
        editor.on('rerender', rerender);
        return () => {
            editor.off('rerender', rerender);
        };
    }, [editor, rerender]);
    const onModeChange = react_1.default.useCallback((type) => {
        editor.changeEditorMode({ mode: type, reason: 'settings' });
        unsetShowPreview();
    }, [editor, unsetShowPreview]);
    const onToolbarVisibilityChange = react_1.default.useCallback((visible) => {
        editor.changeToolbarVisibility({ visible });
    }, [editor]);
    const onSplitModeChange = react_1.default.useCallback((splitModeEnabled) => {
        unsetShowPreview();
        editor.changeSplitModeEnabled({ splitModeEnabled });
    }, [editor, unsetShowPreview]);
    const onShowPreviewChange = react_1.default.useCallback((showPreviewValue) => {
        editor.changeSplitModeEnabled({ splitModeEnabled: false });
        if (showPreviewValue !== showPreview)
            toggleShowPreview();
    }, [editor, showPreview, toggleShowPreview]);
    const editorMode = editor.currentMode;
    const markupSplitMode = editor.splitModeEnabled && editor.splitMode && editorMode === 'markup';
    const canRenderPreview = Boolean(editor.renderPreview && editorMode === 'markup' && !editor.splitModeEnabled);
    (0, react_use_1.useKey)((e) => canRenderPreview && isPreviewKeyDown(e), () => onShowPreviewChange(!showPreview), { event: 'keydown' }, [showPreview, editorMode, onShowPreviewChange, canRenderPreview]);
    const editorWrapperRef = (0, react_1.useRef)(null);
    const splitModeViewWrapperRef = (0, react_1.useRef)(null);
    const settings = (0, react_1.useMemo)(() => (react_1.default.createElement(Settings, { mode: editorMode, onModeChange: onModeChange, toolbarVisibility: editor.toolbarVisible && !showPreview, onToolbarVisibilityChange: onToolbarVisibilityChange, onSplitModeChange: onSplitModeChange, splitModeEnabled: editor.splitModeEnabled, splitMode: editor.splitMode, stickyToolbar: stickyToolbar, onShowPreviewChange: onShowPreviewChange, showPreview: showPreview, renderPreviewButton: canRenderPreview })), [
        canRenderPreview,
        stickyToolbar,
        editor.splitMode,
        editor.splitModeEnabled,
        editor.toolbarVisible,
        editorMode,
        onModeChange,
        onShowPreviewChange,
        onSplitModeChange,
        onToolbarVisibilityChange,
        showPreview,
    ]);
    return (react_1.default.createElement(react_error_boundary_1.ErrorBoundary, { onError: (e) => {
            logger_1.logger.error(e);
        }, fallbackRender: ({ error, resetErrorBoundary }) => {
            toaster.add({
                theme: 'danger',
                name: 'g-md-editor-error',
                title: (0, bundle_1.i18n)('error-title'),
                content: error.message,
            });
            setTimeout(() => {
                resetErrorBoundary();
                editor.changeEditorMode({ mode: 'markup', reason: 'error-boundary' });
            });
            return null;
        } },
        react_1.default.createElement(toaster_1.ToasterContext.Provider, { value: toaster },
            react_1.default.createElement("div", { ref: divRef, className: b({
                    settings: settingsVisible,
                    split: markupSplitMode && editor.splitMode,
                }, [className]) },
                react_1.default.createElement("div", { className: b('editor-wrapper'), ref: editorWrapperRef }, showPreview ? (react_1.default.createElement(react_1.default.Fragment, null,
                    react_1.default.createElement("div", { className: b('preview-wrapper') }, (_b = editor.renderPreview) === null || _b === void 0 ? void 0 : _b.call(editor, {
                        getValue: editor.getValue,
                        mode: 'preview',
                    })),
                    settings)) : (react_1.default.createElement(react_1.default.Fragment, null,
                    editorMode === 'wysiwyg' && (react_1.default.createElement(WysiwygEditorView_1.WysiwygEditorView, { editor: editor, autofocus: autofocus, settingsVisible: settingsVisible, toolbarConfig: wysiwygToolbarConfig, toolbarVisible: editor.toolbarVisible, hiddenActionsConfig: wysiwygHiddenActionsConfig, className: b('editor', { mode: editorMode }), toolbarClassName: b('toolbar'), stickyToolbar: stickyToolbar }, editor.toolbarVisible && settingsVisible && settings)),
                    editorMode === 'markup' && (react_1.default.createElement(MarkupEditorView_1.MarkupEditorView, { editor: editor, autofocus: autofocus, settingsVisible: settingsVisible, toolbarConfig: markupToolbarConfig, toolbarVisible: editor.toolbarVisible, splitMode: editor.splitMode, splitModeEnabled: editor.splitModeEnabled, hiddenActionsConfig: markupHiddenActionsConfig, className: b('editor', { mode: editorMode }), toolbarClassName: b('toolbar'), stickyToolbar: stickyToolbar }, editor.toolbarVisible && settingsVisible && settings)),
                    !editor.toolbarVisible && settingsVisible && settings))),
                markupSplitMode && (react_1.default.createElement(react_1.default.Fragment, null,
                    editor.splitMode === 'horizontal' ? (react_1.default.createElement(HorizontalDrag_1.HorizontalDrag, { editor: editor, isMounted: isMounted, leftElRef: editorWrapperRef, rightElRef: splitModeViewWrapperRef, wrapperRef: divRef })) : (react_1.default.createElement("div", { className: b('resizer') })),
                    react_1.default.createElement(SplitModeView_1.SplitModeView, { editor: editor, ref: splitModeViewWrapperRef })))))));
});
exports.MarkdownEditorView.displayName = 'MarkdownEditorView';
function Settings(props) {
    const wrapperRef = (0, react_1.useRef)(null);
    const isSticky = (0, useSticky_1.useSticky)(wrapperRef) && props.toolbarVisibility && props.stickyToolbar;
    return (react_1.default.createElement("div", { className: b('settings-wrapper') },
        react_1.default.createElement("div", { ref: wrapperRef, className: sticky_1.stickyCn.settings({
                withToolbar: props.toolbarVisibility,
                stickyActive: isSticky,
            }) },
            react_1.default.createElement(settings_1.EditorSettings, Object.assign({}, props)))));
}
function isPreviewKeyDown(e) {
    const modKey = (0, platform_1.isMac)() ? e.metaKey : e.ctrlKey;
    return modKey && e.shiftKey && e.code === 'KeyP';
}
