"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.BundlePreset = void 0;
const behavior_1 = require("../extensions/behavior");
const EditorModeKeymap_1 = require("../extensions/behavior/EditorModeKeymap");
const specs_1 = require("../extensions/specs");
const placeholder_1 = require("../i18n/placeholder");
const commonmark_1 = require("../presets/commonmark");
const default_1 = require("../presets/default");
const full_1 = require("../presets/full");
const yfm_1 = require("../presets/yfm");
const zero_1 = require("../presets/zero");
const shortcuts_1 = require("../shortcuts");
const wysiwyg_1 = require("./config/wysiwyg");
const emoji_1 = require("./emoji");
const DEFAULT_IGNORED_KEYS = ['Tab', 'Shift-Tab'];
const BundlePreset = (builder, opts) => {
    var _a, _b;
    const dropCursor = {
        color: 'var(--g-color-line-brand)',
        width: 2,
    };
    const zeroOptions = Object.assign(Object.assign({}, opts), { cursor: { dropOptions: dropCursor }, clipboard: Object.assign({ pasteFileHandler: opts.fileUploadHandler }, opts.clipboard), selectionContext: Object.assign({ config: wysiwyg_1.wSelectionMenuConfigByPreset.zero }, opts.selectionContext), commandMenu: Object.assign({ actions: wysiwyg_1.wCommandMenuConfigByPreset.zero }, opts.commandMenu), history: Object.assign({ undoKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Undo), redoKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Redo) }, opts.history), baseSchema: Object.assign({ paragraphKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Text), paragraphPlaceholder: (node, parent) => {
                const isDocEmpty = !node.text && (parent === null || parent === void 0 ? void 0 : parent.type.name) === specs_1.BaseNode.Doc && parent.childCount === 1;
                return isDocEmpty ? (0, placeholder_1.i18n)('doc_empty') : null;
            } }, opts.baseSchema) });
    const commonMarkOptions = Object.assign(Object.assign({}, zeroOptions), { selectionContext: Object.assign({ config: wysiwyg_1.wSelectionMenuConfigByPreset.commonmark }, opts.selectionContext), commandMenu: Object.assign({ actions: wysiwyg_1.wCommandMenuConfigByPreset.commonmark }, opts.commandMenu), breaks: Object.assign({ preferredBreak: (opts.mdBreaks ? 'soft' : 'hard') }, opts.breaks), bold: Object.assign({ boldKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Bold) }, opts.bold), italic: Object.assign({ italicKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Italic) }, opts.italic), code: Object.assign({ codeKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Code) }, opts.code), codeBlock: Object.assign({ codeBlockKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.CodeBlock) }, opts.codeBlock), blockquote: Object.assign({ qouteKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Quote) }, opts.blockquote), link: Object.assign({ linkKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Link) }, opts.link), lists: Object.assign({ ulKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.BulletList), olKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.OrderedList), ulInputRules: { plus: false } }, opts.lists) });
    const defaultOptions = Object.assign(Object.assign({}, commonMarkOptions), { selectionContext: Object.assign({ config: wysiwyg_1.wSelectionMenuConfigByPreset.default }, opts.selectionContext), commandMenu: Object.assign({ actions: wysiwyg_1.wCommandMenuConfigByPreset.default }, opts.commandMenu), strike: Object.assign({ strikeKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Strike) }, opts.strike) });
    const yfmOptions = Object.assign(Object.assign({}, defaultOptions), { selectionContext: Object.assign({ config: wysiwyg_1.wSelectionMenuConfigByPreset.yfm }, opts.selectionContext), commandMenu: Object.assign({ actions: wysiwyg_1.wCommandMenuConfigByPreset.yfm }, opts.commandMenu), underline: Object.assign({ underlineKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Underline) }, opts.underline), imgSize: Object.assign({ imageUploadHandler: opts.fileUploadHandler, needToSetDimensionsForUploadedImages: opts.needToSetDimensionsForUploadedImages }, opts.imgSize), checkbox: Object.assign({ checkboxLabelPlaceholder: () => (0, placeholder_1.i18n)('checkbox') }, opts.checkbox), deflist: {
            deflistTermPlaceholder: () => (0, placeholder_1.i18n)('deflist_term'),
            deflistDescPlaceholder: () => (0, placeholder_1.i18n)('deflist_desc'),
        }, yfmCut: Object.assign({ yfmCutKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Cut), yfmCutTitlePlaceholder: () => (0, placeholder_1.i18n)('cut_title'), yfmCutContentPlaceholder: () => (0, placeholder_1.i18n)('cut_content') }, opts.yfmCut), yfmNote: Object.assign({ yfmNoteKey: shortcuts_1.formatter.toPM(shortcuts_1.Action.Note), yfmNoteTitlePlaceholder: () => (0, placeholder_1.i18n)('note_title') }, opts.yfmNote), yfmTable: Object.assign({ yfmTableCellPlaceholder: () => (0, placeholder_1.i18n)('table_cell') }, opts.yfmTable), yfmFile: Object.assign({ fileUploadHandler: opts.fileUploadHandler, needToSetDimensionsForUploadedImages: opts.needToSetDimensionsForUploadedImages }, opts.yfmFile), yfmHeading: Object.assign({ h1Key: shortcuts_1.formatter.toPM(shortcuts_1.Action.Heading1), h2Key: shortcuts_1.formatter.toPM(shortcuts_1.Action.Heading2), h3Key: shortcuts_1.formatter.toPM(shortcuts_1.Action.Heading3), h4Key: shortcuts_1.formatter.toPM(shortcuts_1.Action.Heading4), h5Key: shortcuts_1.formatter.toPM(shortcuts_1.Action.Heading5), h6Key: shortcuts_1.formatter.toPM(shortcuts_1.Action.Heading6), headingPlaceholder: (node) => `${(0, placeholder_1.i18n)('heading')} ${node.attrs[specs_1.YfmHeadingAttr.Level]}` }, opts.yfmHeading), placeholder: {
            [specs_1.YfmNoteNode.NoteContent]: () => (0, placeholder_1.i18n)('note_content'),
        } });
    const fullOptions = Object.assign(Object.assign({}, yfmOptions), { selectionContext: Object.assign({ config: wysiwyg_1.wSelectionMenuConfigByPreset.full }, opts.selectionContext), commandMenu: Object.assign({ actions: wysiwyg_1.wCommandMenuConfigByPreset.full }, opts.commandMenu), emoji: Object.assign({ defs: emoji_1.emojiDefs }, opts.emoji) });
    const zeroIgnoreActions = [shortcuts_1.Action.Undo, shortcuts_1.Action.Redo];
    const commonMarkIgnoreActions = zeroIgnoreActions.concat(shortcuts_1.Action.Bold, shortcuts_1.Action.Italic, shortcuts_1.Action.Code, shortcuts_1.Action.Link, shortcuts_1.Action.Text, shortcuts_1.Action.Heading1, shortcuts_1.Action.Heading2, shortcuts_1.Action.Heading3, shortcuts_1.Action.Heading4, shortcuts_1.Action.Heading5, shortcuts_1.Action.Heading6, shortcuts_1.Action.BulletList, shortcuts_1.Action.OrderedList, shortcuts_1.Action.Quote, shortcuts_1.Action.CodeBlock);
    const defaultIgnoreActions = commonMarkIgnoreActions.concat(shortcuts_1.Action.Strike);
    const yfmIgnoreActions = defaultIgnoreActions.concat(shortcuts_1.Action.Underline, shortcuts_1.Action.Note, shortcuts_1.Action.Cut);
    const fullIgnoreActions = yfmIgnoreActions.concat();
    let ignoreActions;
    switch (opts.preset) {
        case 'zero': {
            ignoreActions = zeroIgnoreActions;
            builder.use(behavior_1.BehaviorPreset, zeroOptions).use(zero_1.ZeroPreset, zeroOptions);
            break;
        }
        case 'commonmark': {
            ignoreActions = commonMarkIgnoreActions;
            builder.use(behavior_1.BehaviorPreset, commonMarkOptions).use(commonmark_1.CommonMarkPreset, commonMarkOptions);
            break;
        }
        case 'default': {
            ignoreActions = defaultIgnoreActions;
            builder.use(behavior_1.BehaviorPreset, defaultOptions).use(default_1.DefaultPreset, defaultOptions);
            break;
        }
        case 'yfm': {
            ignoreActions = yfmIgnoreActions;
            builder.use(behavior_1.BehaviorPreset, yfmOptions).use(yfm_1.YfmPreset, yfmOptions);
            break;
        }
        default: {
            ignoreActions = fullIgnoreActions;
            builder.use(behavior_1.BehaviorPreset, fullOptions).use(full_1.FullPreset, fullOptions);
            break;
        }
    }
    const ignoreKeysList = (_b = (_a = opts.ignoreKeysList) === null || _a === void 0 ? void 0 : _a.slice()) !== null && _b !== void 0 ? _b : [];
    ignoreKeysList.push(...DEFAULT_IGNORED_KEYS);
    for (const action of ignoreActions) {
        const key = shortcuts_1.formatter.toPM(action);
        if (key)
            ignoreKeysList.push(key);
    }
    builder.use(EditorModeKeymap_1.EditorModeKeymap, {
        onSubmit: opts.onSubmit,
        onCancel: opts.onCancel,
        ignoreKeysList,
    });
};
exports.BundlePreset = BundlePreset;
