"use strict";
var _CommandHandler_actions, _CommandHandler_actionStorage, _CommandHandler_nodesIgnoreList, _CommandHandler_filteredActionsCarousel, _CommandHandler_view, _CommandHandler_anchor, _CommandHandler_range, _CommandHandler_filterText, _CommandHandler_popupCloser, _CommandHandler_menuProps, _CommandHandler_menuRenderItem;
Object.defineProperty(exports, "__esModule", { value: true });
exports.CommandHandler = void 0;
const tslib_1 = require("tslib");
const logger_1 = require("../../..//logger");
const autocomplete_popup_1 = require("../../..//utils/autocomplete-popup");
const carousel_1 = require("../../..//utils/carousel");
const lodash_1 = require("../../../lodash");
const Autocomplete_1 = require("../Autocomplete");
const ReactRenderer_1 = require("../ReactRenderer");
const component_1 = require("./component");
const utils_1 = require("./utils");
class CommandHandler {
    constructor({ actions, storage, nodesIgnoreList = [] }) {
        _CommandHandler_actions.set(this, void 0);
        _CommandHandler_actionStorage.set(this, void 0);
        _CommandHandler_nodesIgnoreList.set(this, void 0);
        _CommandHandler_filteredActionsCarousel.set(this, void 0);
        _CommandHandler_view.set(this, void 0);
        _CommandHandler_anchor.set(this, void 0);
        _CommandHandler_range.set(this, void 0);
        _CommandHandler_filterText.set(this, void 0);
        _CommandHandler_popupCloser.set(this, void 0);
        _CommandHandler_menuProps.set(this, void 0);
        _CommandHandler_menuRenderItem.set(this, void 0);
        this.onItemClick = (index) => {
            var _a;
            if (tslib_1.__classPrivateFieldGet(this, _CommandHandler_filteredActionsCarousel, "f")) {
                tslib_1.__classPrivateFieldGet(this, _CommandHandler_filteredActionsCarousel, "f").currentIndex = index;
                this.select();
            }
            (_a = tslib_1.__classPrivateFieldGet(this, _CommandHandler_view, "f")) === null || _a === void 0 ? void 0 : _a.focus();
        };
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_actions, actions, "f");
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_actionStorage, storage, "f");
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_nodesIgnoreList, nodesIgnoreList, "f");
    }
    onOpen(action) {
        this.findAnchor();
        if (!tslib_1.__classPrivateFieldGet(this, _CommandHandler_anchor, "f") || this.shouldIgnore(action)) {
            this.closeAutocomplete(action.view);
            return true;
        }
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_popupCloser, new autocomplete_popup_1.AutocompletePopupCloser(action.view), "f");
        this.updateState(action);
        this.filterActions();
        this.render();
        return true;
    }
    onFilter(action) {
        var _a, _b;
        if (this.shouldIgnore(action) || ((_a = action.filter) === null || _a === void 0 ? void 0 : _a.endsWith('  '))) {
            this.closeAutocomplete(action.view);
            return true;
        }
        this.updateState(action);
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_filterText, (_b = action.filter) === null || _b === void 0 ? void 0 : _b.trim().toLowerCase(), "f");
        const needToClose = this.filterActions();
        this.render();
        if (needToClose) {
            this.closeAutocomplete(action.view);
        }
        return true;
    }
    onArrow(action) {
        if (this.shouldIgnore(action)) {
            (0, Autocomplete_1.closeAutocomplete)(action.view);
            return true;
        }
        this.updateState(action);
        if (!tslib_1.__classPrivateFieldGet(this, _CommandHandler_filteredActionsCarousel, "f"))
            return false;
        switch (action.kind) {
            case Autocomplete_1.AutocompleteActionKind.up: {
                tslib_1.__classPrivateFieldGet(this, _CommandHandler_filteredActionsCarousel, "f").prev();
                break;
            }
            case Autocomplete_1.AutocompleteActionKind.down: {
                tslib_1.__classPrivateFieldGet(this, _CommandHandler_filteredActionsCarousel, "f").next();
                break;
            }
            default:
                return false;
        }
        this.render();
        return true;
    }
    onEnter(action) {
        if (this.shouldIgnore(action)) {
            (0, Autocomplete_1.closeAutocomplete)(action.view);
            return true;
        }
        this.updateState(action);
        this.select();
        return true;
    }
    onClose(action) {
        this.updateState(action);
        this.clear();
        return true;
    }
    onDestroy() {
        this.clear();
    }
    closeAutocomplete(view) {
        setTimeout(() => {
            (0, Autocomplete_1.closeAutocomplete)(view);
        });
    }
    shouldIgnore(action) {
        const parentType = action.view.state.doc.resolve(action.range.from).parent.type;
        return (parentType.spec.commandMenu === false || tslib_1.__classPrivateFieldGet(this, _CommandHandler_nodesIgnoreList, "f").includes(parentType.name));
    }
    select() {
        var _a;
        if (!tslib_1.__classPrivateFieldGet(this, _CommandHandler_view, "f") || !tslib_1.__classPrivateFieldGet(this, _CommandHandler_range, "f"))
            return;
        const action = (_a = tslib_1.__classPrivateFieldGet(this, _CommandHandler_filteredActionsCarousel, "f")) === null || _a === void 0 ? void 0 : _a.currentItem;
        if (!action)
            return;
        const view = tslib_1.__classPrivateFieldGet(this, _CommandHandler_view, "f");
        const range = tslib_1.__classPrivateFieldGet(this, _CommandHandler_range, "f");
        view.dispatch(view.state.tr.deleteRange(range.from, range.to).scrollIntoView());
        action.exec(tslib_1.__classPrivateFieldGet(this, _CommandHandler_actionStorage, "f"));
        view.focus();
        logger_1.logger.action({ mode: 'wysiwyg', source: 'command-menu', action: action.id });
    }
    filterActions() {
        var _a;
        const currentItem = (_a = tslib_1.__classPrivateFieldGet(this, _CommandHandler_filteredActionsCarousel, "f")) === null || _a === void 0 ? void 0 : _a.currentItem;
        const inputText = tslib_1.__classPrivateFieldGet(this, _CommandHandler_filterText, "f");
        const enabledActions = tslib_1.__classPrivateFieldGet(this, _CommandHandler_actions, "f").filter((action) => action.isEnable(tslib_1.__classPrivateFieldGet(this, _CommandHandler_actionStorage, "f")));
        let filteredActions = enabledActions;
        let needToClose = false;
        if (inputText) {
            filteredActions = filterActions(enabledActions, inputText);
            needToClose = !filteredActions.length && needToHide(enabledActions, inputText);
        }
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_filteredActionsCarousel, new carousel_1.ArrayCarousel(filteredActions), "f");
        if (currentItem) {
            const newIndex = tslib_1.__classPrivateFieldGet(this, _CommandHandler_filteredActionsCarousel, "f").array.findIndex((item) => item === currentItem);
            if (newIndex !== -1) {
                tslib_1.__classPrivateFieldGet(this, _CommandHandler_filteredActionsCarousel, "f").currentIndex = newIndex;
            }
        }
        return needToClose;
    }
    render() {
        var _a, _b, _c, _d, _e, _f;
        this.findAnchor();
        const viewItems = (_b = (_a = tslib_1.__classPrivateFieldGet(this, _CommandHandler_filteredActionsCarousel, "f")) === null || _a === void 0 ? void 0 : _a.array) !== null && _b !== void 0 ? _b : [];
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_menuProps, {
            anchor: tslib_1.__classPrivateFieldGet(this, _CommandHandler_anchor, "f"),
            currentIndex: (_c = tslib_1.__classPrivateFieldGet(this, _CommandHandler_filteredActionsCarousel, "f")) === null || _c === void 0 ? void 0 : _c.currentIndex,
            items: viewItems,
            onClick: this.onItemClick,
            onEscapeKeyDown: (_d = tslib_1.__classPrivateFieldGet(this, _CommandHandler_popupCloser, "f")) === null || _d === void 0 ? void 0 : _d.popupEscapeKeyHandler,
            onOutsideClick: (_e = tslib_1.__classPrivateFieldGet(this, _CommandHandler_popupCloser, "f")) === null || _e === void 0 ? void 0 : _e.popupOutsideClickHandler,
        }, "f");
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_menuRenderItem, (_f = tslib_1.__classPrivateFieldGet(this, _CommandHandler_menuRenderItem, "f")) !== null && _f !== void 0 ? _f : this.createMenuRenderItem(), "f");
        tslib_1.__classPrivateFieldGet(this, _CommandHandler_menuRenderItem, "f").rerender();
    }
    updateState({ view, range }) {
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_view, view, "f");
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_range, range, "f");
    }
    clear() {
        var _a, _b;
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_view, undefined, "f");
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_range, undefined, "f");
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_anchor, undefined, "f");
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_filterText, undefined, "f");
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_filteredActionsCarousel, undefined, "f");
        (_a = tslib_1.__classPrivateFieldGet(this, _CommandHandler_popupCloser, "f")) === null || _a === void 0 ? void 0 : _a.cancelTimer();
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_popupCloser, undefined, "f");
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_menuProps, undefined, "f");
        (_b = tslib_1.__classPrivateFieldGet(this, _CommandHandler_menuRenderItem, "f")) === null || _b === void 0 ? void 0 : _b.remove();
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_menuRenderItem, undefined, "f");
    }
    createMenuRenderItem() {
        return (0, ReactRenderer_1.getReactRendererFromState)(tslib_1.__classPrivateFieldGet(this, _CommandHandler_view, "f").state).createItem('command_menu', () => tslib_1.__classPrivateFieldGet(this, _CommandHandler_menuProps, "f") ? (0, component_1.render)(tslib_1.__classPrivateFieldGet(this, _CommandHandler_menuProps, "f")) : null);
    }
    findAnchor() {
        var _a;
        tslib_1.__classPrivateFieldSet(this, _CommandHandler_anchor, (0, utils_1.findDecoElem)((_a = tslib_1.__classPrivateFieldGet(this, _CommandHandler_view, "f")) === null || _a === void 0 ? void 0 : _a.dom), "f");
    }
}
exports.CommandHandler = CommandHandler;
_CommandHandler_actions = new WeakMap(), _CommandHandler_actionStorage = new WeakMap(), _CommandHandler_nodesIgnoreList = new WeakMap(), _CommandHandler_filteredActionsCarousel = new WeakMap(), _CommandHandler_view = new WeakMap(), _CommandHandler_anchor = new WeakMap(), _CommandHandler_range = new WeakMap(), _CommandHandler_filterText = new WeakMap(), _CommandHandler_popupCloser = new WeakMap(), _CommandHandler_menuProps = new WeakMap(), _CommandHandler_menuRenderItem = new WeakMap();
function filterActions(actions, text) {
    return actions.filter((action) => action.id.toLowerCase().includes(text) ||
        ((0, lodash_1.isFunction)(action.title) ? action.title() : action.title).toLowerCase().includes(text));
}
const CHARS_TO_HIDE = 4;
function needToHide(actions, text) {
    let iter = 1;
    do {
        const prevInput = text.slice(0, text.length - iter);
        const prevActions = filterActions(actions, prevInput);
        if (prevActions.length)
            break;
        iter++;
    } while (iter < text.length && iter < CHARS_TO_HIDE);
    return iter >= CHARS_TO_HIDE;
}
