"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Code = exports.code = exports.codeType = exports.codeMarkName = void 0;
const tslib_1 = require("tslib");
const prosemirror_codemark_1 = tslib_1.__importDefault(require("prosemirror-codemark"));
const prosemirror_commands_1 = require("prosemirror-commands");
const prosemirror_state_1 = require("prosemirror-state");
const actions_1 = require("../../../utils/actions");
const keymap_1 = require("../../../utils/keymap");
const CodeSpecs_1 = require("./CodeSpecs");
var CodeSpecs_2 = require("./CodeSpecs");
Object.defineProperty(exports, "codeMarkName", { enumerable: true, get: function () { return CodeSpecs_2.codeMarkName; } });
Object.defineProperty(exports, "codeType", { enumerable: true, get: function () { return CodeSpecs_2.codeType; } });
/** @deprecated Use `codeMarkName` instead */
exports.code = CodeSpecs_1.codeMarkName;
const codeAction = 'code';
const Code = (builder, opts) => {
    builder.use(CodeSpecs_1.CodeSpecs);
    builder.addAction(codeAction, ({ schema }) => (0, actions_1.createToggleMarkAction)((0, CodeSpecs_1.codeType)(schema)));
    if (opts === null || opts === void 0 ? void 0 : opts.codeKey) {
        const { codeKey } = opts;
        builder.addKeymap(({ schema }) => ({
            [codeKey]: (0, keymap_1.withLogAction)('code_inline', (0, prosemirror_commands_1.toggleMark)((0, CodeSpecs_1.codeType)(schema))),
        }));
    }
    builder
        // codemark adds inputRule for `code`,
        // adds fake cursor behaviour when crossing inline_code boundaries,
        // wrap current selected text to inline_code when '`' was pressed.
        // See demo: https://curvenote.github.io/prosemirror-codemark/
        .addPlugin(({ schema }) => (0, prosemirror_codemark_1.default)({ markType: (0, CodeSpecs_1.codeType)(schema) }));
    builder.addPlugin(() => 
    // apply codemark when typing text between ``
    new prosemirror_state_1.Plugin({
        props: {
            handleTextInput: (view, from, to, text) => {
                var _a, _b, _c, _d;
                const { selection: { $anchor }, tr, schema, } = view.state;
                if (((_b = (_a = $anchor.nodeBefore) === null || _a === void 0 ? void 0 : _a.text) === null || _b === void 0 ? void 0 : _b.endsWith('`')) &&
                    ((_d = (_c = $anchor.nodeAfter) === null || _c === void 0 ? void 0 : _c.text) === null || _d === void 0 ? void 0 : _d.startsWith('`'))) {
                    view.dispatch(tr.replaceRangeWith(from - 1, to + 1, view.state.schema.text(text, [(0, CodeSpecs_1.codeType)(schema).create()])));
                    return true;
                }
                return false;
            },
        },
    }));
};
exports.Code = Code;
