"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.CodeBlock = exports.codeBlockType = exports.codeBlockLangAttr = exports.CodeBlockNodeAttr = exports.codeBlockNodeName = exports.resetCodeblock = void 0;
const prosemirror_commands_1 = require("prosemirror-commands");
const prosemirror_model_1 = require("prosemirror-model");
const prosemirror_utils_1 = require("prosemirror-utils");
const lodash_1 = require("../../../lodash");
const inputrules_1 = require("../../../utils/inputrules");
const keymap_1 = require("../../../utils/keymap");
const CodeBlockHighlight_1 = require("./CodeBlockHighlight/CodeBlockHighlight");
const CodeBlockSpecs_1 = require("./CodeBlockSpecs");
const commands_1 = require("./commands");
const const_1 = require("./const");
const codeBlockPastePlugin_1 = require("./plugins/codeBlockPastePlugin");
var commands_2 = require("./commands");
Object.defineProperty(exports, "resetCodeblock", { enumerable: true, get: function () { return commands_2.resetCodeblock; } });
var CodeBlockSpecs_2 = require("./CodeBlockSpecs");
Object.defineProperty(exports, "codeBlockNodeName", { enumerable: true, get: function () { return CodeBlockSpecs_2.codeBlockNodeName; } });
Object.defineProperty(exports, "CodeBlockNodeAttr", { enumerable: true, get: function () { return CodeBlockSpecs_2.CodeBlockNodeAttr; } });
Object.defineProperty(exports, "codeBlockLangAttr", { enumerable: true, get: function () { return CodeBlockSpecs_2.codeBlockLangAttr; } });
Object.defineProperty(exports, "codeBlockType", { enumerable: true, get: function () { return CodeBlockSpecs_2.codeBlockType; } });
const CodeBlock = (builder, opts) => {
    var _a;
    builder.use(CodeBlockSpecs_1.CodeBlockSpecs, opts);
    builder.addKeymap((deps) => {
        const { codeBlockKey } = opts;
        const bindings = { Enter: prosemirror_commands_1.newlineInCode, Backspace: commands_1.resetCodeblock };
        if (codeBlockKey) {
            bindings[codeBlockKey] = (0, keymap_1.withLogAction)('code_block', (0, prosemirror_commands_1.setBlockType)((0, const_1.cbType)(deps.schema)));
        }
        return bindings;
    });
    builder.addInputRules(({ schema }) => ({ rules: [codeBlockRule((0, const_1.cbType)(schema))] }));
    builder.addAction(const_1.cbAction, ({ schema, serializer }) => {
        const cb = (0, const_1.cbType)(schema);
        const cmd = (state, dispatch) => {
            if (!(0, prosemirror_commands_1.setBlockType)(cb)(state))
                return false;
            if (dispatch) {
                const markup = serializer.serialize(prosemirror_model_1.Fragment.from(state.selection.content().content));
                dispatch(state.tr.replaceSelectionWith(cb.createAndFill({}, markup ? state.schema.text(markup) : null)));
            }
            return true;
        };
        return {
            isActive: (state) => (0, prosemirror_utils_1.hasParentNodeOfType)(cb)(state.selection),
            isEnable: cmd,
            run: cmd,
        };
    });
    builder.addPlugin(codeBlockPastePlugin_1.codeBlockPastePlugin, builder.Priority.High);
    if ((0, lodash_1.isFunction)(opts.langs)) {
        builder.use(opts.langs);
    }
    else {
        builder.use(CodeBlockHighlight_1.CodeBlockHighlight, (_a = opts.langs) !== null && _a !== void 0 ? _a : {});
    }
};
exports.CodeBlock = CodeBlock;
// Given a code block node type, returns an input rule that turns a
// textblock starting with three backticks into a code block.
function codeBlockRule(nodeType) {
    return (0, inputrules_1.textblockTypeInputRule)(/^```$/, nodeType);
}
