"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.mergeListsPlugin = void 0;
const prosemirror_state_1 = require("prosemirror-state");
const prosemirror_utils_1 = require("prosemirror-utils");
const utils_1 = require("../utils");
const mergeListsPlugin = () => new prosemirror_state_1.Plugin({
    appendTransaction(trs, oldState, newState) {
        const docChanged = trs.some((tr) => tr.docChanged);
        if (!docChanged)
            return null;
        const hasParentList = (0, prosemirror_utils_1.hasParentNode)(utils_1.isListNode)(newState.selection) ||
            (0, prosemirror_utils_1.hasParentNode)(utils_1.isListNode)(oldState.selection);
        if (!hasParentList)
            return null;
        const { tr } = newState;
        const listNodes = (0, prosemirror_utils_1.findChildren)(tr.doc, utils_1.isListNode, true);
        mergeAdjacentNodesWithSameType(tr, listNodes);
        return tr.docChanged ? tr : null;
    },
});
exports.mergeListsPlugin = mergeListsPlugin;
function mergeAdjacentNodesWithSameType(tr, nodes) {
    for (let i = nodes.length - 1; i > 0; i--) {
        const prev = nodes[i - 1];
        const next = nodes[i];
        if (prev.node.type === next.node.type && prev.pos + prev.node.nodeSize === next.pos) {
            tr.join(next.pos);
        }
    }
}
