"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.WMermaidNodeView = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const react_dom_1 = require("react-dom");
const behavior_1 = require("../../../behavior");
const const_1 = require("../MermaidSpecs/const");
const MermaidView_1 = require("./MermaidView");
let mermaidInstance;
class WMermaidNodeView {
    constructor(node, view, getPos, opts) {
        this.getMermaidInstance = () => mermaidInstance;
        const { loadRuntimeScript } = opts;
        this.node = node;
        this.dom = document.createElement('div');
        this.dom.classList.add('mermaid-container');
        this.dom.contentEditable = 'false';
        this.view = view;
        this.getPos = getPos;
        this.loadRuntimeScript = loadRuntimeScript;
        this.initializeMermaid();
        this.renderItem = (0, behavior_1.getReactRendererFromState)(view.state).createItem('mermaid-view', this.renderMermaid.bind(this));
    }
    initializeMermaid() {
        // https://github.com/diplodoc-platform/mermaid-extension/tree/master#prepared-mermaid-runtime
        window.mermaidJsonp = window.mermaidJsonp || [];
        window.mermaidJsonp.push((mermaid) => {
            mermaidInstance = mermaid;
        });
        this.loadRuntimeScript();
    }
    update(node) {
        if (node.type !== this.node.type)
            return false;
        this.node = node;
        this.renderItem.rerender();
        return true;
    }
    destroy() {
        this.renderItem.remove();
    }
    ignoreMutation() {
        return true;
    }
    stopEvent(e) {
        const target = e.target;
        if (typeof target.className === 'string' &&
            target.className.includes('prosemirror-stop-event')) {
            return true;
        }
        return false;
    }
    onChange(attrs) {
        const pos = this.getPos();
        if (pos === undefined)
            return;
        const tr = this.view.state.tr.setNodeMarkup(pos, undefined, Object.assign(Object.assign({}, this.node.attrs), attrs), []);
        this.view.dispatch(tr);
    }
    renderMermaid() {
        return (0, react_dom_1.createPortal)(react_1.default.createElement(MermaidView_1.MermaidView, { view: this.view, onChange: this.onChange.bind(this), node: this.node, getMermaidInstance: this.getMermaidInstance, getPos: this.getPos }), this.dom);
    }
}
exports.WMermaidNodeView = WMermaidNodeView;
