"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeWidget = exports.addWidget = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const WidgetDecoration_1 = require("../../../behavior/WidgetDecoration");
const markdown_1 = require("../../../markdown");
const upload_1 = require("../YfmFilePaste/upload");
const YfmFileSpecs_1 = require("../YfmFileSpecs");
const view_1 = require("./view");
const addWidget = (tr, deps, uploadFiles) => {
    return new FileWidgetDescriptor(tr.selection.from, deps, uploadFiles).applyTo(tr);
};
exports.addWidget = addWidget;
exports.removeWidget = WidgetDecoration_1.removeDecoration;
class FileWidgetDescriptor extends WidgetDecoration_1.ReactWidgetDescriptor {
    constructor(initPos, deps, uploadFiles) {
        super(initPos, 'file_placeholder');
        this.widgetHandler = null;
        this.uploadFiles = uploadFiles;
        this.domElem = document.createElement('span');
        this.deps = deps;
    }
    getWidgetHandler(view, getPos) {
        if (!this.widgetHandler) {
            this.widgetHandler = new FileWidgetHandler({
                view,
                getPos,
                decoId: this.id,
                uploadFiles: this.uploadFiles,
            }, this.deps);
        }
        return this.widgetHandler;
    }
    getDomElem() {
        return this.domElem;
    }
    renderReactElement(view, getPos) {
        return this.getWidgetHandler(view, getPos).renderWidgetView(view, getPos);
    }
    remove() {
        super.remove();
        this.widgetHandler = null;
    }
}
class FileWidgetHandler {
    constructor({ decoId, view, getPos, uploadFiles }, deps) {
        this.onCancel = () => {
            this.view.dispatch((0, WidgetDecoration_1.removeDecoration)(this.view.state.tr, this.decoId));
            this.view.focus();
        };
        this.onSubmit = (params) => {
            var _a;
            if (!params.src || !params.name)
                return;
            const href = (_a = this.normalizeUrl(params.src)) === null || _a === void 0 ? void 0 : _a.url;
            const download = params.name;
            if (!href)
                return;
            const node = (0, YfmFileSpecs_1.fileType)(this.view.state.schema).create({ href, download });
            this.insertNodes([node]);
        };
        this.onAttach = async (files) => {
            if (!this.uploadFiles)
                return;
            const { view } = this;
            new YfmFileWidgetUploadProcess(view, files, this.uploadFiles, this.getPos()).run();
            view.dispatch((0, exports.removeWidget)(view.state.tr, this.decoId));
            view.focus();
        };
        this.decoId = decoId;
        this.view = view;
        this.getPos = getPos;
        this.uploadFiles = uploadFiles;
        this.normalizeUrl = (0, markdown_1.normalizeUrlFactory)(deps);
    }
    renderWidgetView(view, getPos) {
        this.view = view;
        this.getPos = getPos;
        return (react_1.default.createElement(view_1.FilePlaceholder, { onCancel: this.onCancel, onSubmit: this.onSubmit, onAttach: this.uploadFiles && this.onAttach }));
    }
    insertNodes(fileNodes) {
        let tr = this.view.state.tr;
        tr = tr.insert(tr.mapping.map(this.getPos()), fileNodes);
        tr = (0, exports.removeWidget)(tr, this.decoId);
        this.view.dispatch(tr);
        this.view.focus();
    }
}
class YfmFileWidgetUploadProcess extends upload_1.YfmFilesUploadProcessBase {
    constructor(view, files, uploadHandler, position) {
        super(view, files, uploadHandler);
        this.skeletonInitPos = position;
    }
    getSkeletonInitPos() {
        return this.skeletonInitPos;
    }
}
