"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.YfmHtmlBlockView = exports.generateID = exports.cnHelper = exports.cnYfmHtmlBlock = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importStar(require("react"));
const icons_1 = require("@gravity-ui/icons");
const uikit_1 = require("@gravity-ui/uikit");
const debounce_1 = tslib_1.__importDefault(require("lodash/debounce"));
const classname_1 = require("../../../../classname");
const TextInput_1 = require("../../../../forms/TextInput");
const common_1 = require("../../../../i18n/common");
const hooks_1 = require("../../../../react-utils/hooks");
const remove_node_1 = require("../../../../utils/remove-node");
const const_1 = require("../YfmHtmlBlockSpecs/const");
exports.cnYfmHtmlBlock = (0, classname_1.cn)('yfm-html-block');
exports.cnHelper = (0, classname_1.cn)('yfm-html-block-helper');
const b = exports.cnYfmHtmlBlock;
function generateID() {
    return Math.random().toString(36).substr(2, 8);
}
exports.generateID = generateID;
const DEFAULT_PADDING = 20;
const DEFAULT_DELAY = 100;
const YfmHtmlBlockPreview = ({ html, onСlick, config }) => {
    var _a, _b, _c, _d, _e, _f;
    const ref = (0, react_1.useRef)(null);
    const styles = (0, react_1.useRef)({});
    const classNames = (0, react_1.useRef)([]);
    const resizeConfig = (0, react_1.useRef)({});
    const [height, setHeight] = (0, react_1.useState)('100%');
    (0, react_1.useEffect)(() => {
        var _a, _b;
        resizeConfig.current = {
            padding: (_a = config === null || config === void 0 ? void 0 : config.resizePadding) !== null && _a !== void 0 ? _a : DEFAULT_PADDING,
            delay: (_b = config === null || config === void 0 ? void 0 : config.resizeDelay) !== null && _b !== void 0 ? _b : DEFAULT_DELAY,
        };
        setStyles(config === null || config === void 0 ? void 0 : config.styles);
        setClassNames(config === null || config === void 0 ? void 0 : config.classNames);
    }, [config, (_c = (_b = (_a = ref.current) === null || _a === void 0 ? void 0 : _a.contentWindow) === null || _b === void 0 ? void 0 : _b.document) === null || _c === void 0 ? void 0 : _c.body]);
    const handleLoadIFrame = () => {
        var _a;
        const contentWindow = (_a = ref.current) === null || _a === void 0 ? void 0 : _a.contentWindow;
        handleResizeIFrame();
        if (contentWindow) {
            const frameDocument = contentWindow.document;
            frameDocument.addEventListener('dblclick', () => {
                onСlick();
            });
        }
    };
    const handleResizeIFrame = () => {
        var _a, _b;
        if (ref.current) {
            const contentWindow = (_a = ref.current) === null || _a === void 0 ? void 0 : _a.contentWindow;
            if (contentWindow) {
                const body = contentWindow.document.body;
                if (body) {
                    const height = body.scrollHeight +
                        (((_b = resizeConfig.current) === null || _b === void 0 ? void 0 : _b.padding) || DEFAULT_PADDING) +
                        'px';
                    setHeight(height);
                }
            }
        }
    };
    const setClassNames = (newClassNames) => {
        var _a, _b;
        const body = (_b = (_a = ref.current) === null || _a === void 0 ? void 0 : _a.contentWindow) === null || _b === void 0 ? void 0 : _b.document.body;
        if (body && newClassNames) {
            const previousClassNames = classNames.current;
            // remove all classes that were in previousClassNames but are not in classNames
            previousClassNames.forEach((className) => {
                if (!newClassNames.includes(className)) {
                    body.classList.remove(className);
                }
            });
            // add classes that are in classNames
            newClassNames.forEach((className) => {
                if (!body.classList.contains(className)) {
                    body.classList.add(className);
                }
            });
            classNames.current = newClassNames;
        }
    };
    const setStyles = (newStyles) => {
        var _a, _b;
        const body = (_b = (_a = ref.current) === null || _a === void 0 ? void 0 : _a.contentWindow) === null || _b === void 0 ? void 0 : _b.document.body;
        if (body && newStyles) {
            const previousStyles = styles.current;
            // remove all styles that are in previousStyles but not in styles
            Object.keys(previousStyles).forEach((property) => {
                if (!Object.prototype.hasOwnProperty.call(newStyles, property)) {
                    body.style.removeProperty(property);
                }
            });
            // sdd or update styles that are in styles
            Object.keys(newStyles).forEach((property) => {
                body.style.setProperty(property, newStyles[property]);
            });
            // update current styles to the new styles
            styles.current = newStyles;
        }
    };
    (0, react_1.useEffect)(() => {
        var _a;
        (_a = ref.current) === null || _a === void 0 ? void 0 : _a.addEventListener('load', handleLoadIFrame);
        return () => {
            var _a;
            (_a = ref.current) === null || _a === void 0 ? void 0 : _a.removeEventListener('load', handleLoadIFrame);
        };
    }, [html]);
    (0, react_1.useEffect)(() => {
        var _a, _b;
        if (ref.current) {
            const resizeObserver = new window.ResizeObserver((0, debounce_1.default)(handleResizeIFrame, (_b = (_a = resizeConfig.current) === null || _a === void 0 ? void 0 : _a.delay) !== null && _b !== void 0 ? _b : DEFAULT_DELAY));
            resizeObserver.observe(ref.current);
        }
    }, [(_f = (_e = (_d = ref.current) === null || _d === void 0 ? void 0 : _d.contentWindow) === null || _e === void 0 ? void 0 : _e.document) === null || _f === void 0 ? void 0 : _f.body]);
    return (react_1.default.createElement("iframe", { style: {
            height,
        }, ref: ref, title: generateID(), frameBorder: 0, className: b('content'), srcDoc: html }));
};
const CodeEditMode = ({ initialText, onSave, onCancel }) => {
    const [text, setText] = (0, react_1.useState)(initialText || '\n');
    return (react_1.default.createElement("div", { className: b({ editing: true }) },
        react_1.default.createElement("div", { className: b('editor') },
            react_1.default.createElement(TextInput_1.TextAreaFixed, { controlProps: {
                    className: (0, exports.cnHelper)({ 'prosemirror-stop-event': true }),
                }, value: text, onUpdate: (v) => {
                    setText(v);
                }, autoFocus: true }),
            react_1.default.createElement("div", { className: b('controls') },
                react_1.default.createElement("div", null,
                    react_1.default.createElement(uikit_1.Button, { onClick: onCancel, view: 'flat' },
                        react_1.default.createElement("span", { className: (0, exports.cnHelper)({ 'prosemirror-stop-event': true }) }, (0, common_1.i18n)('cancel'))),
                    react_1.default.createElement(uikit_1.Button, { onClick: () => onSave(text), view: 'action' },
                        react_1.default.createElement("span", { className: (0, exports.cnHelper)({ 'prosemirror-stop-event': true }) }, (0, common_1.i18n)('save'))))))));
};
const YfmHtmlBlockView = ({ onChange, node, getPos, view, useConfig }) => {
    const [editing, setEditing, unsetEditing, toggleEditing] = (0, hooks_1.useBooleanState)(Boolean(node.attrs[const_1.YfmHtmlBlockConsts.NodeAttrs.newCreated]));
    const config = useConfig === null || useConfig === void 0 ? void 0 : useConfig();
    const [menuOpen, , , toggleMenuOpen] = (0, hooks_1.useBooleanState)(false);
    const buttonRef = (0, react_1.useRef)(null);
    const handleClick = () => {
        setEditing();
    };
    if (editing) {
        return (react_1.default.createElement(CodeEditMode, { initialText: node.attrs[const_1.YfmHtmlBlockConsts.NodeAttrs.srcdoc], onCancel: unsetEditing, onSave: (v) => {
                onChange({ [const_1.YfmHtmlBlockConsts.NodeAttrs.srcdoc]: v });
                unsetEditing();
            } }));
    }
    return (react_1.default.createElement("div", { className: b(), onDoubleClick: setEditing },
        react_1.default.createElement(uikit_1.Label, { className: b('label'), icon: react_1.default.createElement(uikit_1.Icon, { size: 16, data: icons_1.Eye }) }, (0, common_1.i18n)('preview')),
        react_1.default.createElement(YfmHtmlBlockPreview, { html: node.attrs[const_1.YfmHtmlBlockConsts.NodeAttrs.srcdoc], "on\u0421lick": handleClick, config: config }),
        react_1.default.createElement("div", { className: b('menu') },
            react_1.default.createElement(uikit_1.Button, { onClick: toggleMenuOpen, ref: buttonRef, size: 's', className: (0, exports.cnHelper)({ 'prosemirror-stop-event': true }) },
                react_1.default.createElement(uikit_1.Icon, { data: icons_1.Ellipsis, className: (0, exports.cnHelper)({ 'prosemirror-stop-event': true }) })),
            react_1.default.createElement(uikit_1.Popup, { anchorRef: buttonRef, open: menuOpen, onClose: toggleMenuOpen, placement: ['bottom-end'] },
                react_1.default.createElement(uikit_1.Menu, null,
                    react_1.default.createElement(uikit_1.Menu.Item, { onClick: () => {
                            toggleEditing();
                            toggleMenuOpen();
                        } }, (0, common_1.i18n)('edit')),
                    react_1.default.createElement(uikit_1.Menu.Item, { onClick: () => {
                            const pos = getPos();
                            if (pos === undefined)
                                return;
                            (0, remove_node_1.removeNode)({
                                node,
                                pos,
                                tr: view.state.tr,
                                dispatch: view.dispatch,
                            });
                        } }, (0, common_1.i18n)('remove')))))));
};
exports.YfmHtmlBlockView = YfmHtmlBlockView;
