"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.FileForm = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const uikit_1 = require("@gravity-ui/uikit");
const classname_1 = require("../classname");
const forms_1 = require("../i18n/forms");
const lodash_1 = require("../lodash");
const TextInput_1 = require("./TextInput");
const base_1 = tslib_1.__importDefault(require("./base"));
const components_1 = require("./components");
const utils_1 = require("./utils");
const b = (0, classname_1.cn)('file-form');
const FileForm = ({ className, autoFocus, onCancel, onSubmit, onAttach, loading, }) => {
    const [tabId, setTabId] = react_1.default.useState(() => (0, lodash_1.isFunction)(onAttach) ? "attach" /* TabId.Attach */ : "link" /* TabId.Link */);
    const [src, setSrc] = react_1.default.useState('');
    const [name, setName] = react_1.default.useState('');
    const shouldRenderTabs = (0, lodash_1.isFunction)(onAttach);
    react_1.default.useLayoutEffect(() => {
        if (!shouldRenderTabs && tabId === "attach" /* TabId.Attach */) {
            setTabId("link" /* TabId.Link */);
        }
    }, [shouldRenderTabs, tabId]);
    const handleSubmit = () => {
        onSubmit({
            src: src.trim(),
            name: name.trim(),
        });
    };
    const inputEnterKeyHandler = (0, utils_1.enterKeyHandler)(handleSubmit);
    return (react_1.default.createElement(base_1.default.Form, { className: b(null, [className]) },
        shouldRenderTabs && (react_1.default.createElement(uikit_1.Tabs, { activeTab: tabId, onSelectTab: setTabId, items: [
                { id: "attach" /* TabId.Attach */, title: (0, forms_1.i18n)('common_tab_attach') },
                { id: "link" /* TabId.Link */, title: (0, forms_1.i18n)('common_tab_link') },
            ] })),
        tabId === "attach" /* TabId.Attach */ && onAttach && (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement(base_1.default.Layout, null, (0, forms_1.i18n)('file_upload_help')),
            react_1.default.createElement(base_1.default.Footer, { onCancel: onCancel },
                react_1.default.createElement(components_1.ButtonAttach, { multiple: true, buttonProps: {
                        size: 's',
                        view: 'action',
                        loading,
                    }, onUpdate: onAttach }, (0, forms_1.i18n)('common_action_upload'))))),
        tabId === "link" /* TabId.Link */ && (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement(base_1.default.Layout, null,
                react_1.default.createElement(base_1.default.Row, { label: (0, forms_1.i18n)('common_link'), help: (0, forms_1.i18n)('file_link_help'), control: react_1.default.createElement(TextInput_1.TextInputFixed, { size: "s", view: "normal", value: src, onUpdate: setSrc, placeholder: "https://", autoFocus: autoFocus, className: b('input', { type: 'link' }), onKeyPress: inputEnterKeyHandler }) }),
                react_1.default.createElement(base_1.default.Row, { label: (0, forms_1.i18n)('file_name'), control: react_1.default.createElement(uikit_1.TextInput, { size: "s", view: "normal", value: name, onUpdate: setName, className: b('input', { type: 'name' }), onKeyPress: inputEnterKeyHandler }) })),
            react_1.default.createElement(base_1.default.Footer, { onCancel: onCancel, onSubmit: handleSubmit })))));
};
exports.FileForm = FileForm;
