"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ImageForm = void 0;
const tslib_1 = require("tslib");
const react_1 = tslib_1.__importDefault(require("react"));
const uikit_1 = require("@gravity-ui/uikit");
const classname_1 = require("../classname");
const forms_1 = require("../i18n/forms");
const lodash_1 = require("../lodash");
const TextInput_1 = require("./TextInput");
const base_1 = tslib_1.__importDefault(require("./base"));
const components_1 = require("./components");
const utils_1 = require("./utils");
const b = (0, classname_1.cn)('image-form');
const ImageForm = ({ className, autoFocus, onCancel, onSubmit, onAttach, loading, }) => {
    const [tabId, setTabId] = react_1.default.useState(() => (0, lodash_1.isFunction)(onAttach) ? "attach" /* ImageTabId.Attach */ : "link" /* ImageTabId.Link */);
    const [url, setUrl] = react_1.default.useState('');
    const [name, setName] = react_1.default.useState('');
    const [alt, setAlt] = react_1.default.useState('');
    const [width, setWidth] = react_1.default.useState();
    const [height, setHeight] = react_1.default.useState();
    const shouldRenderTabs = (0, lodash_1.isFunction)(onAttach);
    react_1.default.useLayoutEffect(() => {
        if (!shouldRenderTabs && tabId === "attach" /* ImageTabId.Attach */) {
            setTabId("link" /* ImageTabId.Link */);
        }
    }, [shouldRenderTabs, tabId]);
    const handleSubmit = () => {
        const data = {
            url: url.trim(),
            name: name.trim(),
            alt: alt.trim(),
        };
        if (typeof width === 'number' && width >= 0)
            data.width = width;
        if (typeof height === 'number' && height >= 0)
            data.height = height;
        onSubmit(data);
    };
    const inputEnterKeyHandler = (0, utils_1.enterKeyHandler)(handleSubmit);
    return (react_1.default.createElement(base_1.default.Form, { className: b(null, [className]) },
        shouldRenderTabs && (react_1.default.createElement(uikit_1.Tabs, { activeTab: tabId, onSelectTab: setTabId, items: [
                { id: "attach" /* ImageTabId.Attach */, title: (0, forms_1.i18n)('common_tab_attach') },
                { id: "link" /* ImageTabId.Link */, title: (0, forms_1.i18n)('common_tab_link') },
            ] })),
        tabId === "attach" /* ImageTabId.Attach */ && onAttach && (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement(base_1.default.Layout, null, (0, forms_1.i18n)('image_upload_help')),
            react_1.default.createElement(base_1.default.Footer, { onCancel: onCancel },
                react_1.default.createElement(components_1.ButtonAttach, { multiple: true, accept: "image/*", onUpdate: onAttach, buttonProps: { size: 's', view: 'action', loading } }, (0, forms_1.i18n)('common_action_upload'))))),
        tabId === "link" /* ImageTabId.Link */ && (react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement(base_1.default.Layout, null,
                react_1.default.createElement(base_1.default.Row, { label: (0, forms_1.i18n)('common_link'), control: react_1.default.createElement(TextInput_1.TextInputFixed, { size: "s", view: "normal", value: url, onUpdate: setUrl, placeholder: "https://", autoFocus: autoFocus, className: b('input', { type: 'link' }), onKeyPress: inputEnterKeyHandler }) }),
                react_1.default.createElement(base_1.default.Row, { label: (0, forms_1.i18n)('image_name'), control: react_1.default.createElement(uikit_1.TextInput, { size: "s", view: "normal", value: name, onUpdate: setName, className: b('input', { type: 'name' }), onKeyPress: inputEnterKeyHandler }) }),
                react_1.default.createElement(base_1.default.Row, { label: (0, forms_1.i18n)('image_alt'), help: (0, forms_1.i18n)('image_alt_help'), control: react_1.default.createElement(uikit_1.TextInput, { size: "s", view: "normal", value: alt, onUpdate: setAlt, className: b('input', { type: 'alt' }), onKeyPress: inputEnterKeyHandler }) }),
                react_1.default.createElement(base_1.default.Row, { label: (0, forms_1.i18n)('common_sizes'), control: react_1.default.createElement("div", { className: b('size-controls') },
                        react_1.default.createElement(components_1.NumberInput, { min: 0, size: "s", view: "normal", value: width, onUpdate: setWidth, placeholder: (0, forms_1.i18n)('image_size_width'), className: b('input', { type: 'width' }), onKeyDown: inputEnterKeyHandler }),
                        "x",
                        react_1.default.createElement(components_1.NumberInput, { min: 0, size: "s", view: "normal", value: height, onUpdate: setHeight, placeholder: (0, forms_1.i18n)('image_size_height'), className: b('input', { type: 'height' }), onKeyDown: inputEnterKeyHandler })) })),
            react_1.default.createElement(base_1.default.Footer, { onCancel: onCancel, onSubmit: handleSubmit })))));
};
exports.ImageForm = ImageForm;
