"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.removeColumn = void 0;
const __1 = require("..");
const utils_1 = require("../utils");
const removeColumn = (state, dispatch, _, attrs = {}) => {
    const { tablePos, columnNumber } = attrs;
    if (typeof tablePos !== 'number' || typeof columnNumber !== 'number')
        return false;
    if (Number.isNaN(tablePos) || Number.isNaN(columnNumber))
        return false;
    if (tablePos < 0 || columnNumber < 0)
        return false;
    const tableNode = state.doc.nodeAt(tablePos);
    if (!tableNode || tableNode.nodeSize <= 2 || !(0, __1.isTableNode)(tableNode))
        return false;
    const tableBodyNode = tableNode.firstChild;
    if (!tableBodyNode || tableBodyNode.nodeSize <= 2 || !(0, utils_1.isTableBodyNode)(tableBodyNode))
        return false;
    // there is one column left
    if (tableBodyNode.firstChild && tableBodyNode.firstChild.childCount < 2)
        return false;
    if (dispatch) {
        const { tr } = state;
        tableBodyNode.forEach((rowNode, rowOffset) => {
            if (!(0, utils_1.isTableRowNode)(rowNode))
                return;
            rowNode.forEach((cellNode, cellOffset, cellIndex) => {
                if (!(0, utils_1.isTableCellNode)(cellNode))
                    return;
                if (cellIndex === columnNumber) {
                    // table -> tbody -> tr -> td
                    const from = tablePos + 2 + rowOffset + 1 + cellOffset;
                    const to = from + cellNode.nodeSize;
                    tr.delete(tr.mapping.map(from), tr.mapping.map(to));
                }
            });
        });
        dispatch(tr);
    }
    return true;
};
exports.removeColumn = removeColumn;
