import type { ReactNode } from 'react';
import type { Extension as CodemirrorExtension } from '@codemirror/state';
import { CommonEditor, MarkupString } from '../common';
import { WysiwygEditorOptions } from '../core';
import { ReactRenderStorage } from '../extensions';
import { Receiver } from '../utils/event-emitter';
import type { FileUploadHandler } from '../utils/upload';
export declare type EditorMode = 'wysiwyg' | 'markup';
export declare type SplitMode = false | 'horizontal' | 'vertical';
export declare type EditorPreset = 'zero' | 'commonmark' | 'default' | 'yfm' | 'full';
export declare type RenderPreview = ({ getValue, mode, }: {
    getValue: () => MarkupString;
    mode: 'preview' | 'split';
}) => ReactNode;
export declare type ToolbarActionData = {
    editorMode: EditorMode;
    id: string;
    attrs?: {
        [key: string]: any;
    };
};
interface EventMap {
    change: null;
    cancel: null;
    submit: null;
    'toolbar-action': ToolbarActionData;
    'change-editor-mode': {
        mode: EditorMode;
    };
    'change-toolbar-visibility': {
        visible: boolean;
    };
    'change-split-mode-enabled': {
        splitModeEnabled: boolean;
    };
}
export interface Editor extends Receiver<EventMap>, CommonEditor {
    readonly currentMode: EditorMode;
    readonly toolbarVisible: boolean;
    setEditorMode(mode: EditorMode): void;
    moveCursor(position: 'start' | 'end' | {
        line: number;
    }): void;
}
export declare type EditorOptions = Pick<WysiwygEditorOptions, 'allowHTML' | 'linkify' | 'linkifyTlds' | 'extensions'> & {
    initialMarkup?: MarkupString;
    /** @default 'wysiwyg' */
    initialEditorMode?: EditorMode;
    /** @default true */
    initialToolbarVisible?: boolean;
    /** @default false
     * Has no effect if splitMode is false or undefined
     */
    initialSplitModeEnabled?: boolean;
    renderStorage: ReactRenderStorage;
    fileUploadHandler?: FileUploadHandler;
    /**
     * If we need to set dimensions for uploaded images
     *
     * @default false
     */
    needToSetDimensionsForUploadedImages?: boolean;
    /**
     * Called before switching from the markup editor to the wysiwyg editor.
     * You can use it to pre-process the value from the markup editor before it gets into the wysiwyg editor.
     */
    prepareRawMarkup?: (value: MarkupString) => MarkupString;
    splitMode?: SplitMode;
    renderPreview?: RenderPreview;
    preset: EditorPreset;
    extraMarkupExtensions?: CodemirrorExtension[];
};
export {};
