import type { PresetName } from 'markdown-it';
import type { CommonEditor, MarkupString } from '../common';
import type { ActionStorage } from './types/actions';
import type { Extension } from './types/extension';
import type { Parser } from './types/parser';
import type { Serializer } from './types/serializer';
declare type OnChange = (editor: WysiwygEditor) => void;
export declare type WysiwygEditorOptions = {
    domElem?: Element;
    /** markdown markup */
    initialContent?: string;
    extensions?: Extension;
    /** @default 'default' */
    mdPreset?: PresetName;
    allowHTML?: boolean;
    linkify?: boolean;
    linkifyTlds?: string | string[];
    /** Call on any state change (move cursor, change selection, etc...) */
    onChange?: OnChange;
    /** Call only if document change */
    onDocChange?: OnChange;
};
export declare class WysiwygEditor implements CommonEditor, ActionStorage {
    #private;
    get dom(): HTMLElement;
    get serializer(): Serializer;
    get parser(): Parser;
    get actions(): WysiwygEditor.Actions;
    constructor({ domElem, initialContent, extensions, allowHTML, mdPreset, linkify, linkifyTlds, onChange, onDocChange, }: WysiwygEditorOptions);
    action<T extends keyof WysiwygEditor.Actions>(actionName: T): WysiwygEditor.Actions[T];
    focus(): void;
    hasFocus(): boolean;
    getValue(): MarkupString;
    isEmpty(): boolean;
    clear(): void;
    replace(newMarkup: MarkupString): void;
    prepend(markup: MarkupString): void;
    append(markup: MarkupString): void;
    moveCursor(position: 'start' | 'end'): void;
    destroy(): void;
    private tryOnChange;
}
declare global {
    namespace WysiwygEditor {
        interface Actions {
        }
    }
}
export {};
