import { newlineInCode, setBlockType } from 'prosemirror-commands';
import { Fragment } from 'prosemirror-model';
import { hasParentNodeOfType } from 'prosemirror-utils';
import { isFunction } from '../../../lodash';
import { textblockTypeInputRule } from '../../../utils/inputrules';
import { withLogAction } from '../../../utils/keymap';
import { CodeBlockHighlight } from './CodeBlockHighlight/CodeBlockHighlight';
import { CodeBlockSpecs } from './CodeBlockSpecs';
import { resetCodeblock } from './commands';
import { cbAction, cbType } from './const';
import { codeBlockPastePlugin } from './plugins/codeBlockPastePlugin';
export { resetCodeblock } from './commands';
export { codeBlockNodeName, CodeBlockNodeAttr, codeBlockLangAttr, codeBlockType, } from './CodeBlockSpecs';
export const CodeBlock = (builder, opts) => {
    var _a;
    builder.use(CodeBlockSpecs, opts);
    builder.addKeymap((deps) => {
        const { codeBlockKey } = opts;
        const bindings = { Enter: newlineInCode, Backspace: resetCodeblock };
        if (codeBlockKey) {
            bindings[codeBlockKey] = withLogAction('code_block', setBlockType(cbType(deps.schema)));
        }
        return bindings;
    });
    builder.addInputRules(({ schema }) => ({ rules: [codeBlockRule(cbType(schema))] }));
    builder.addAction(cbAction, ({ schema, serializer }) => {
        const cb = cbType(schema);
        const cmd = (state, dispatch) => {
            if (!setBlockType(cb)(state))
                return false;
            if (dispatch) {
                const markup = serializer.serialize(Fragment.from(state.selection.content().content));
                dispatch(state.tr.replaceSelectionWith(cb.createAndFill({}, markup ? state.schema.text(markup) : null)));
            }
            return true;
        };
        return {
            isActive: (state) => hasParentNodeOfType(cb)(state.selection),
            isEnable: cmd,
            run: cmd,
        };
    });
    builder.addPlugin(codeBlockPastePlugin, builder.Priority.High);
    if (isFunction(opts.langs)) {
        builder.use(opts.langs);
    }
    else {
        builder.use(CodeBlockHighlight, (_a = opts.langs) !== null && _a !== void 0 ? _a : {});
    }
};
// Given a code block node type, returns an input rule that turns a
// textblock starting with three backticks into a code block.
function codeBlockRule(nodeType) {
    return textblockTypeInputRule(/^```$/, nodeType);
}
