import React from 'react';
import isNumber from 'is-number';
import { ReactWidgetDescriptor, removeDecoration } from '../../../behavior/WidgetDecoration';
import { imageType, normalizeUrlFactory } from '../../../markdown';
import { ImgSizeAttr } from '../../../specs';
import { ImagesUploadProcess } from '../ImagePaste/upload';
import { FilePlaceholder } from './view';
export const addWidget = (tr, deps, opts) => {
    return new ImageWidgetDescriptor(tr.selection.from, deps, opts).applyTo(tr);
};
export const removeWidget = removeDecoration;
class ImageWidgetDescriptor extends ReactWidgetDescriptor {
    constructor(initPos, deps, opts) {
        super(initPos, 'image_placeholder');
        this.widgetHandler = null;
        this.domElem = document.createElement('span');
        this.deps = deps;
        this.uploadImages = opts.uploadImages;
        this.needToSetDimensionsForUploadedImages = opts.needToSetDimensionsForUploadedImages;
    }
    getWidgetHandler(view, getPos) {
        if (!this.widgetHandler) {
            this.widgetHandler = new ImageWidgetHandler({
                view,
                getPos,
                decoId: this.id,
                uploadImages: this.uploadImages,
                needToSetDimensionsForUploadedImages: this.needToSetDimensionsForUploadedImages,
            }, this.deps);
        }
        return this.widgetHandler;
    }
    getDomElem() {
        return this.domElem;
    }
    renderReactElement(view, getPos) {
        return this.getWidgetHandler(view, getPos).renderWidgetView(view, getPos);
    }
    remove() {
        var _a;
        super.remove();
        (_a = this.widgetHandler) === null || _a === void 0 ? void 0 : _a.destruct();
        this.widgetHandler = null;
    }
}
class ImageWidgetHandler {
    constructor({ decoId, view, getPos, uploadImages, needToSetDimensionsForUploadedImages, }, deps) {
        this.cancelled = false;
        this.onCancel = () => {
            this.cancelled = true;
            this.view.dispatch(removeDecoration(this.view.state.tr, this.decoId));
            this.view.focus();
        };
        this.onSubmit = (params) => {
            var _a;
            if (this.cancelled)
                return;
            const url = (_a = this.normalizeUrl(params.url)) === null || _a === void 0 ? void 0 : _a.url;
            if (!url)
                return;
            const attrs = {
                [ImgSizeAttr.Src]: url,
                [ImgSizeAttr.Title]: params.name,
                [ImgSizeAttr.Alt]: params.alt,
                [ImgSizeAttr.Width]: isNumber(params.width) ? String(params.width) : null,
                [ImgSizeAttr.Height]: isNumber(params.height) ? String(params.height) : null,
            };
            const node = imageType(this.view.state.schema).create(attrs);
            this.insertNodes([node]);
        };
        this.onAttach = async (files) => {
            if (this.cancelled || !this.uploadImages)
                return;
            const { view } = this;
            new ImagesUploadProcess(view, files, this.uploadImages, this.getPos(), {
                needDimmensions: this.needToSetDimensionsForUploadedImages,
            }).run();
            view.dispatch(removeWidget(view.state.tr, this.decoId));
            view.focus();
        };
        this.decoId = decoId;
        this.view = view;
        this.getPos = getPos;
        this.uploadImages = uploadImages;
        this.normalizeUrl = normalizeUrlFactory(deps);
        this.needToSetDimensionsForUploadedImages = needToSetDimensionsForUploadedImages;
    }
    destruct() {
        this.cancelled = true;
    }
    renderWidgetView(view, getPos) {
        this.view = view;
        this.getPos = getPos;
        return (React.createElement(FilePlaceholder, { onCancel: this.onCancel, onSubmit: this.onSubmit, onAttach: this.uploadImages && this.onAttach }));
    }
    insertNodes(fileNodes) {
        if (this.cancelled)
            return;
        let tr = this.view.state.tr;
        tr = tr.insert(tr.mapping.map(this.getPos()), fileNodes);
        tr = removeWidget(tr, this.decoId);
        this.view.dispatch(tr);
        this.view.focus();
    }
}
