import React, { useEffect, useRef, useState } from 'react';
import { Ellipsis as DotsIcon } from '@gravity-ui/icons';
import { Button, Icon, Loader, Menu, Popup } from '@gravity-ui/uikit';
import { cn } from '../../../../classname';
import { TextAreaFixed as TextArea } from '../../../../forms/TextInput';
import { useBooleanState } from '../../../../react-utils/hooks';
import { removeNode } from '../../../../utils/remove-node';
import { MermaidConsts } from '../MermaidSpecs/const';
export const cnMermaid = cn('Mermaid');
export const cnDiagramHelper = cn('MermaidHelper');
import './Mermaid.css';
const b = cnMermaid;
const MermaidPreview = ({ mermaidInstance, text = '', }) => {
    const [svg, setSvg] = useState();
    const [error, setError] = useState(null);
    useEffect(() => {
        const p = async () => {
            if (mermaidInstance) {
                try {
                    // Validates syntax and throws error if text is invalid
                    await mermaidInstance.parse(text);
                    const { svg: S } = await mermaidInstance.render(`mermaid-${Date.now()}`, text);
                    setSvg(S);
                    setError(null);
                }
                catch (e) {
                    setError(e.message);
                }
            }
        };
        p();
    }, [mermaidInstance, text]);
    if (error) {
        return React.createElement("div", { className: b('Error') }, error && React.createElement("div", null, error));
    }
    return (React.createElement("div", { className: b('Preview') }, svg ? React.createElement("div", { className: "mermaid", dangerouslySetInnerHTML: { __html: svg } }) : React.createElement(Loader, null)));
};
const DiagramEditMode = ({ initialText, onSave, onCancel, mermaidInstance }) => {
    const [text, setText] = useState(initialText || '');
    return (React.createElement("div", { className: b() },
        React.createElement(MermaidPreview, { mermaidInstance: mermaidInstance, text: text }),
        React.createElement("div", { className: b('Editor') },
            React.createElement("div", null,
                React.createElement(TextArea, { controlProps: {
                        className: cnDiagramHelper({ 'prosemirror-stop-event': true }),
                    }, value: text, onUpdate: (v) => {
                        setText(v);
                    }, autoFocus: true })),
            React.createElement("div", { className: b('Controls') },
                React.createElement("div", null,
                    React.createElement(Button, { onClick: onCancel, view: 'flat' },
                        React.createElement("span", { className: cnDiagramHelper({ 'prosemirror-stop-event': true }) }, "Cancel")),
                    React.createElement(Button, { onClick: () => onSave(text), view: 'action' },
                        React.createElement("span", { className: cnDiagramHelper({ 'prosemirror-stop-event': true }) }, "Save")))))));
};
export const MermaidView = ({ onChange, node, getPos, view, getMermaidInstance }) => {
    const [mermaidInstance, setMermaidInstance] = useState(null);
    const [editing, setEditing, unsetEditing, toggleEditing] = useBooleanState(Boolean(node.attrs[MermaidConsts.NodeAttrs.newCreated]));
    const [menuOpen, , , toggleMenuOpen] = useBooleanState(false);
    const buttonRef = useRef(null);
    useEffect(() => {
        const waitForMermaid = () => setTimeout(() => {
            const instance = getMermaidInstance();
            if (instance) {
                setMermaidInstance(instance);
                return;
            }
            waitForMermaid();
        }, 100);
        waitForMermaid();
    }, []);
    if (editing) {
        return (React.createElement(DiagramEditMode, { initialText: node.attrs[MermaidConsts.NodeAttrs.content], mermaidInstance: mermaidInstance, onCancel: unsetEditing, onSave: (v) => {
                onChange({ [MermaidConsts.NodeAttrs.content]: v });
                unsetEditing();
            } }));
    }
    return (React.createElement("div", { className: b(), onDoubleClick: setEditing },
        React.createElement(MermaidPreview, { mermaidInstance: mermaidInstance, text: node.attrs[MermaidConsts.NodeAttrs.content] }),
        React.createElement("div", null,
            React.createElement(Button, { onClick: toggleMenuOpen, ref: buttonRef, size: 's', className: cnDiagramHelper({ 'prosemirror-stop-event': true }) },
                React.createElement(Icon, { data: DotsIcon, className: cnDiagramHelper({ 'prosemirror-stop-event': true }) })),
            React.createElement(Popup, { anchorRef: buttonRef, open: menuOpen, onClose: toggleMenuOpen, placement: ['bottom-end'] },
                React.createElement(Menu, null,
                    React.createElement(Menu.Item, { onClick: () => {
                            toggleEditing();
                            toggleMenuOpen();
                        } }, "Edit"),
                    React.createElement(Menu.Item, { onClick: () => {
                            const pos = getPos();
                            if (pos === undefined)
                                return;
                            removeNode({
                                node,
                                pos,
                                tr: view.state.tr,
                                dispatch: view.dispatch,
                            });
                        } }, "Remove"))))));
};
