import React from 'react';
import { createPortal } from 'react-dom';
import { getReactRendererFromState } from '../../../behavior';
import { YfmHtmlBlockConsts } from '../YfmHtmlBlockSpecs/const';
import { YfmHtmlBlockView } from './YfmHtmlBlockView';
export class WYfmHtmlBlockNodeView {
    constructor({ node, view, getPos, options, }) {
        this.options = {};
        this.node = node;
        this.dom = document.createElement('div');
        this.dom.classList.add('yfm-html-block-container');
        this.dom.contentEditable = 'false';
        this.view = view;
        this.getPos = getPos;
        this.options = options;
        this.renderItem = getReactRendererFromState(view.state).createItem('yfmHtmlBlock-view', this.renderYfmHtmlBlock.bind(this));
    }
    update(node) {
        if (node.type !== this.node.type)
            return false;
        if (node.attrs[YfmHtmlBlockConsts.NodeAttrs.newCreated] !==
            this.node.attrs[YfmHtmlBlockConsts.NodeAttrs.newCreated])
            return false;
        this.node = node;
        this.renderItem.rerender();
        return true;
    }
    destroy() {
        this.renderItem.remove();
    }
    ignoreMutation() {
        return true;
    }
    stopEvent(e) {
        const target = e.target;
        if (typeof target.className === 'string' &&
            target.className.includes('prosemirror-stop-event')) {
            return true;
        }
        return false;
    }
    onChange(attrs) {
        const pos = this.getPos();
        if (pos === undefined)
            return;
        const tr = this.view.state.tr.setNodeMarkup(pos, undefined, Object.assign(Object.assign({}, this.node.attrs), attrs), []);
        this.view.dispatch(tr);
    }
    renderYfmHtmlBlock() {
        return createPortal(React.createElement(YfmHtmlBlockView, { useConfig: this.options.useConfig, view: this.view, onChange: this.onChange.bind(this), node: this.node, getPos: this.getPos }), this.dom);
    }
}
